(* structure Main: The main program. 
   Builds and connects lexers and parsers,
   implements a simple top-level evaluation loop (Main.main).
   
   For most experiments with the implementation, it should not be
   necessary to change this file.
*)

structure Main =
struct

open PPUtil
open Syntax
open Core

(* 
 * We apply the functors generated from arith.lex and arith.grm to produce
 * the ArithParser structure.
 *)

  structure ArithLrVals =
    ArithLrValsFun(structure Token = LrParser.Token)

  structure ArithLex =
    ArithLexFun(structure Tokens = ArithLrVals.Tokens)

  structure ArithParser =
    Join(structure LrParser = LrParser
	 structure ParserData = ArithLrVals.ParserData
	 structure Lex = ArithLex)

(* 
 * We need a function which given a lexer invokes the parser. The
 * function invoke does this.
 *)

  fun invoke lexstream =
      let fun print_error (s,i:int,_) =
	      TextIO.output(TextIO.stdOut,
			    "Error, line " ^ (Int.toString i) ^ ", " ^ s ^ "\n")
       in ArithParser.parse(0,lexstream,print_error,())
      end

(* 
 * Finally, we need a driver function that reads one or more expressions
 * from the standard input. The function main, shown below, does
 * this. It runs the evaluator on the standard input and terminates when
 * an end-of-file is encountered.
 *)

  (* dummy tokens *)
  val dummyEOF = ArithLrVals.Tokens.EOF(0,0)
  val dummySEMI = ArithLrVals.Tokens.SEMI(0,0)

  fun main (src: Source.inputSource) = 
      let 
          val lexer = ArithParser.makeLexer (fn _ => Source.getLine src)
	  fun loop lexer =
	      let val (result,lexer) = invoke lexer
		  val (nextToken,lexer) = ArithParser.Stream.get lexer
	       in case result
		    of SOME(Syntax.Eval t) =>
                         (print_string "value: "; printtm(eval t))
		     | SOME(Syntax.Import f) =>
                         (main (Source.newSource(f,TextIO.openIn f,false,())))
		     | NONE => (print_string "nothing"; print_newline());
	          if ArithParser.sameToken(nextToken,dummyEOF) then
                       Source.closeSource src
		  else loop lexer
	      end
       in loop lexer
      end

fun interact () =
    main(Source.newSource("stdIn",TextIO.stdIn,true,()))

end (* structure Main *)
