(* syntax.sml *)
(* abstract syntax for arith *)

structure Syntax =
struct

(* ---------------------------------------------------------------------- *)
(* Datatypes *)

type info = SourceMap.region option

datatype term
  = TmTrue of info
  | TmFalse of info
  | TmIf of info * term * term * term
  | TmZero of info
  | TmSucc of info * term
  | TmPred of info *term
  | TmIsZero of info * term

(* top level commands *)
datatype command
  = Import of string
  | Eval of term

(* translating from nonnegative integers to terms.
 * only the outermost term constructor inherits the file info *)
fun intToTerm(info: info, n) =
    let fun trans 0 = TmZero(NONE)
          | trans n = TmSucc(NONE,trans(n-1))
     in case n
          of 0 => TmZero(info)
           | n => TmSucc(info,trans(n-1))
    end

(* Extracting file info *)

fun tmInfo t =
    case t
      of TmTrue(fi) => fi
       | TmFalse(fi) => fi
       | TmIf(fi,_,_,_) => fi
       | TmZero(fi) => fi
       | TmSucc(fi,_) => fi
       | TmPred(fi,_) => fi
       | TmIsZero(fi,_) => fi 

(* ---------------------------------------------------------------------- *)

(* The printing functions call these utility functions to insert grouping
  information and line-breaking hints for the pretty-printing library:
     obox   Open a "box" whose contents will be indented by two spaces if
            the whole box cannot fit on the current line
     obox0  Same but indent continuation lines to the same column as the
            beginning of the box rather than 2 more columns to the right
     cbox   Close the current box
     break  Insert a breakpoint indicating where the line maybe broken if
            necessary.
  See the documentation for the Format module in the OCaml library for
  more details. 
*)

open PPUtil

fun obox0() = open_hvbox 0
fun obox() = open_hvbox 2
fun cbox() = close_box()
fun break() = print_break 0 0
val pr = print_string

fun printtm_Term outer t = 
    case t 
      of TmIf(_,t1, t2, t3) =>
         (obox0();
          pr "if ";
	  printtm_Term false t1;
	  print_space();
	  print_string "then ";
	  printtm_Term false t2;
	  print_space();
	  pr "else ";
	  printtm_Term false t3;
	  cbox())
       | t => printtm_AppTerm outer t

and printtm_AppTerm outer t =
    case t
      of TmPred(_,t1) =>
         (pr "pred "; printtm_ATerm false t1)
       | TmIsZero(_,t1) =>
         (pr "iszero "; printtm_ATerm false t1)
       | t => printtm_ATerm outer t

and printtm_ATerm outer t =
    case t
      of TmTrue _ => pr "true"
       | TmFalse _ => pr "false"
       | TmZero _ => pr "0"
       | TmSucc (_,t1) =>
          let fun f (n, TmZero _) = pr (Int.toString n)
                | f (n, TmSucc (_,s)) = f (n+1, s)
                | f (n,_) = (pr "(succ "; printtm_ATerm false t1; pr ")")
           in f (1,t1) end
       | t => (pr "("; printtm_Term outer t; pr ")")

fun printtm t = (printtm_Term true t; print_newline())

end (* structure Syntax *)
